/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.announce.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.announce.domain.DmAnnounce;
import me.zhengjie.modules.dm.announce.service.DmAnnounceService;
import me.zhengjie.modules.dm.announce.service.dto.DmAnnounceQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Li Rui
* @date 2021-11-30
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "系统公告管理")
@RequestMapping("/api/dmAnnounce")
public class DmAnnounceController {

    private final DmAnnounceService dmAnnounceService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmAnnounce:list')")
    public void download(HttpServletResponse response, DmAnnounceQueryCriteria criteria) throws IOException {
        dmAnnounceService.download(dmAnnounceService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询系统公告")
    @ApiOperation("查询系统公告")
    @PreAuthorize("@el.check('dmAnnounce:list')")
    public ResponseEntity<Object> query(DmAnnounceQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmAnnounceService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增系统公告")
    @ApiOperation("新增系统公告")
    @PreAuthorize("@el.check('dmAnnounce:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmAnnounce resources){
        return new ResponseEntity<>(dmAnnounceService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改系统公告")
    @ApiOperation("修改系统公告")
    @PreAuthorize("@el.check('dmAnnounce:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmAnnounce resources){
        dmAnnounceService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除系统公告")
    @ApiOperation("删除系统公告")
    @PreAuthorize("@el.check('dmAnnounce:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Integer[] ids) {
        dmAnnounceService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Log("取消系统公告")
    @ApiOperation("取消系统公告")
    @GetMapping(value = "/cancel")
    public ResponseEntity<Object> cancel(Integer id){
        dmAnnounceService.cancel(id);
        return new ResponseEntity<>(true,HttpStatus.OK);
    }
}