/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.api.domain;

import lombok.Data;
import cn.hutool.core.bean.BeanUtil;
import io.swagger.annotations.ApiModelProperty;
import cn.hutool.core.bean.copier.CopyOptions;
import me.zhengjie.base.BaseEntity;
import me.zhengjie.domain.LocalStorage;
import me.zhengjie.modules.dm.system.domain.DmSystem;

import javax.persistence.*;
import javax.validation.constraints.*;
import java.sql.Timestamp;
import java.io.Serializable;
import java.util.List;

/**
* @website https://el-admin.vip
* @description /
* @author Li Rui
* @date 2021-11-30
**/
@Entity
@Data
@Table(name="dm_api")
public class DmApi extends BaseEntity implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    @ApiModelProperty(value = "id")
    private Integer id;

    @Column(name = "api_name")
    @ApiModelProperty(value = "接口名称")
    private String apiName;

    @Column(name = "api_url")
    @ApiModelProperty(value = "接口地址")
    private String apiUrl;

    @ManyToMany
    @ApiModelProperty(value = "子系统列表")
    @JoinTable(name = "dm_system_api",
            joinColumns = {@JoinColumn(name = "api_id",referencedColumnName = "id")},
            inverseJoinColumns = {@JoinColumn(name = "system_id",referencedColumnName = "id")})
    private List<DmSystem> dmSystems;


    /**
     * status 状态  0启用  1 禁用
     */
    @Column(name = "status")
    @ApiModelProperty(value = " status 状态  0启用  1 禁用")
    private String status;


//    @Column(name = "storage_id")
//    @ApiModelProperty(value = "接口配置文件id")
//    private Long storageId;

    @OneToOne
    @JoinColumn(name = "storage_id")
    @ApiModelProperty(value = "接口配置文件")
    private LocalStorage localStorage;

//    @Column(name = "create_by")
//    @ApiModelProperty(value = "创建者")
//    private String createBy;
//
//    @Column(name = "update_by")
//    @ApiModelProperty(value = "更新者")
//    private String updateBy;
//
//    @Column(name = "create_time")
//    @ApiModelProperty(value = "创建时间")
//    private Timestamp createTime;
//
//    @Column(name = "update_time")
//    @ApiModelProperty(value = "更新时间")
//    private Timestamp updateTime;

    public void copy(DmApi source){
        BeanUtil.copyProperties(source,this, CopyOptions.create().setIgnoreNullValue(true));
    }
}