/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.api.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.api.domain.DmApi;
import me.zhengjie.modules.dm.api.service.DmApiService;
import me.zhengjie.modules.dm.api.service.dto.DmApiQueryCriteria;
import me.zhengjie.modules.dm.system.domain.DmSystem;
import me.zhengjie.modules.dm.system.service.DmSystemService;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Li Rui
* @date 2021-11-30
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "接口管理")
@RequestMapping("/api/dmApi")
public class DmApiController {

    private final DmApiService dmApiService;

    private final DmSystemService dmSystemService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmApi:list')")
    public void download(HttpServletResponse response, DmApiQueryCriteria criteria) throws IOException {
        dmApiService.download(dmApiService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询接口")
    @ApiOperation("查询接口")
    @PreAuthorize("@el.check('dmApi:list')")
    public ResponseEntity<Object> query(DmApiQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmApiService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增接口")
    @ApiOperation("新增接口")
    @PreAuthorize("@el.check('dmApi:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmApi resources){
        return new ResponseEntity<>(dmApiService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改接口")
    @ApiOperation("修改接口")
    @PreAuthorize("@el.check('dmApi:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmApi resources){
        dmApiService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除接口")
    @ApiOperation("删除接口")
    @PreAuthorize("@el.check('dmApi:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Integer[] ids) {
        dmApiService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Log("根据子系统查询接口")
    @ApiOperation("根据子系统查询接口")
    @GetMapping(value = "/getDmApiBySystem")
    @PreAuthorize("@el.check('dmApi:list')")
    public ResponseEntity<Object> getDmApiBySystem(Integer systemId) {
        return new ResponseEntity<>(dmSystemService.findById(systemId),HttpStatus.OK);
    }

    @Log("根据子系统查询接口")
    @ApiOperation("根据子系统查询接口")
    @GetMapping(value = "/findByIdAndStatus")
    @PreAuthorize("@el.check('dmApi:list')")
    public ResponseEntity<Object> findByIdAndStatus(@RequestParam("systemId") Integer systemId,@RequestParam("status")Integer status) {
        return new ResponseEntity<>(dmApiService.findByIdAndStatus(systemId,status),HttpStatus.OK);
    }

}