/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.api.service.impl;

import me.zhengjie.modules.dm.api.domain.DmApi;
import me.zhengjie.utils.ValidationUtil;
import me.zhengjie.utils.FileUtil;
import lombok.RequiredArgsConstructor;
import me.zhengjie.modules.dm.api.repository.DmApiRepository;
import me.zhengjie.modules.dm.api.service.DmApiService;
import me.zhengjie.modules.dm.api.service.dto.DmApiDto;
import me.zhengjie.modules.dm.api.service.dto.DmApiQueryCriteria;
import me.zhengjie.modules.dm.api.service.mapstruct.DmApiMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import me.zhengjie.utils.PageUtil;
import me.zhengjie.utils.QueryHelp;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.LinkedHashMap;

/**
* @website https://el-admin.vip
* @description 服务实现
* @author Li Rui
* @date 2021-11-30
**/
@Service
@RequiredArgsConstructor
public class DmApiServiceImpl implements DmApiService {

    private final DmApiRepository dmApiRepository;
    private final DmApiMapper dmApiMapper;

    @Override
    public Map<String,Object> queryAll(DmApiQueryCriteria criteria, Pageable pageable){
        Page<DmApi> page = dmApiRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder),pageable);
        return PageUtil.toPage(page.map(dmApiMapper::toDto));
    }

    @Override
    public List<DmApiDto> queryAll(DmApiQueryCriteria criteria){
        return dmApiMapper.toDto(dmApiRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder)));
    }

    @Override
    @Transactional
    public DmApiDto findById(Integer id) {
        DmApi dmApi = dmApiRepository.findById(id).orElseGet(DmApi::new);
        ValidationUtil.isNull(dmApi.getId(),"DmApi","id",id);
        return dmApiMapper.toDto(dmApi);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DmApiDto create(DmApi resources) {
        return dmApiMapper.toDto(dmApiRepository.save(resources));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DmApi resources) {
        DmApi dmApi = dmApiRepository.findById(resources.getId()).orElseGet(DmApi::new);
        ValidationUtil.isNull( dmApi.getId(),"DmApi","id",resources.getId());
        dmApi.copy(resources);
        dmApiRepository.save(dmApi);
    }

    @Override
    public void deleteAll(Integer[] ids) {
        for (Integer id : ids) {
            dmApiRepository.deleteById(id);
        }
    }

    @Override
    public void download(List<DmApiDto> all, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (DmApiDto dmApi : all) {
            Map<String,Object> map = new LinkedHashMap<>();
            map.put("接口名称", dmApi.getApiName());
            map.put("接口地址", dmApi.getApiUrl());
            map.put("接口配置文件id", dmApi.getStorageId());
            map.put("创建者", dmApi.getCreateBy());
            map.put("更新者", dmApi.getUpdateBy());
            map.put("创建时间", dmApi.getCreateTime());
            map.put("更新时间", dmApi.getUpdateTime());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }

    @Override
    public List<DmApiDto> findByIdAndStatus(Integer systemId, Integer status) {
        return  dmApiMapper.toDto(dmApiRepository.findByIdAndStatus(systemId,status));
    }
}