/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.appMenu.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.appMenu.domain.DmAppMenu;
import me.zhengjie.modules.dm.appMenu.service.DmAppMenuService;
import me.zhengjie.modules.dm.appMenu.service.dto.DmAppMenuQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Li Rui
* @date 2021-12-21
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "app菜单管理")
@RequestMapping("/api/dmAppMenu")
public class DmAppMenuController {

    private final DmAppMenuService dmAppMenuService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmAppMenu:list')")
    public void download(HttpServletResponse response, DmAppMenuQueryCriteria criteria) throws IOException {
        dmAppMenuService.download(dmAppMenuService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询app菜单")
    @ApiOperation("查询app菜单")
    @PreAuthorize("@el.check('dmAppMenu:list')")
    public ResponseEntity<Object> query(DmAppMenuQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmAppMenuService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增app菜单")
    @ApiOperation("新增app菜单")
    @PreAuthorize("@el.check('dmAppMenu:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmAppMenu resources){
        return new ResponseEntity<>(dmAppMenuService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改app菜单")
    @ApiOperation("修改app菜单")
    @PreAuthorize("@el.check('dmAppMenu:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmAppMenu resources){
        dmAppMenuService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除app菜单")
    @ApiOperation("删除app菜单")
    @PreAuthorize("@el.check('dmAppMenu:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Integer[] ids) {
        dmAppMenuService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}