/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.card.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.card.domain.DmCard;
import me.zhengjie.modules.dm.card.service.DmCardService;
import me.zhengjie.modules.dm.card.service.dto.DmCardQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Li Rui
* @date 2021-12-31
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "卡号管理")
@RequestMapping("/api/dmCard")
public class DmCardController {

    private final DmCardService dmCardService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmCard:list')")
    public void download(HttpServletResponse response, DmCardQueryCriteria criteria) throws IOException {
        dmCardService.download(dmCardService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询卡号")
    @ApiOperation("查询卡号")
    @PreAuthorize("@el.check('dmCard:list')")
    public ResponseEntity<Object> query(DmCardQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmCardService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增卡号")
    @ApiOperation("新增卡号")
    @PreAuthorize("@el.check('dmCard:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmCard resources){
        return new ResponseEntity<>(dmCardService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改卡号")
    @ApiOperation("修改卡号")
    @PreAuthorize("@el.check('dmCard:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmCard resources){
        dmCardService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除卡号")
    @ApiOperation("删除卡号")
    @PreAuthorize("@el.check('dmCard:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Integer[] ids) {
        dmCardService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}