/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.information.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.information.domain.DmInformation;
import me.zhengjie.modules.dm.information.service.DmInformationService;
import me.zhengjie.modules.dm.information.service.dto.DmInformationQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author xwx
* @date 2021-04-22
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "dmInformation管理")
@RequestMapping("/api/dmInformation")
public class DmInformationController {

    private final DmInformationService dmInformationService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmInformation:list')")
    public void download(HttpServletResponse response, DmInformationQueryCriteria criteria) throws IOException {
        dmInformationService.download(dmInformationService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询dmInformation")
    @ApiOperation("查询dmInformation")
    @PreAuthorize("@el.check('dmInformation:list')")
    public ResponseEntity<Object> query(DmInformationQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmInformationService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @GetMapping(value = "approvalList")
    @Log("查询dmInformation")
    @ApiOperation("查询dmInformation")
    @PreAuthorize("@el.check('dmInformation:list')")
    public ResponseEntity<Object> approvalList(DmInformationQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmInformationService.findAllDSP(pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增dmInformation")
    @ApiOperation("新增dmInformation")
    @PreAuthorize("@el.check('dmInformation:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmInformation resources){
        return new ResponseEntity<>(dmInformationService.create(resources),HttpStatus.CREATED);
    }

    @PostMapping(value = "/addApprovalUser")
    @Log("新增审批人")
    @ApiOperation("新增审批人")
    @PreAuthorize("@el.check('dmInformation:edit')")
    public ResponseEntity<Object> addApprovalUser(@Validated @RequestBody DmInformation resources){
        dmInformationService.addApprovalUser(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除dmInformation")
    @ApiOperation("删除dmInformation")
    @PreAuthorize("@el.check('dmInformation:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmInformationService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PutMapping
    @Log("修改dmInformation")
    @ApiOperation("修改dmInformation")
    @PreAuthorize("@el.check('dmInformation:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmInformation resources){
        dmInformationService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @PostMapping(value = "/approval")
    @Log("审批")
    @ApiOperation("审批")
    public ResponseEntity<Object> Approval(@RequestBody DmInformation resources){
        dmInformationService.approval(resources);
        return new ResponseEntity<>("审批完成",HttpStatus.OK);


    }

    @Log("撤回dmInformation")
    @ApiOperation("撤回dmInformation")
    @PreAuthorize("@el.check('dmInformation:edit')")
    @PostMapping(value = "ch")
    public ResponseEntity<Object> ch(@RequestBody Long ids) {
        dmInformationService.ch(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}