/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.information.service;

import me.zhengjie.modules.dm.information.domain.DmInformation;
import me.zhengjie.modules.dm.information.service.dto.DmInformationDto;
import me.zhengjie.modules.dm.information.service.dto.DmInformationQueryCriteria;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author xwx
* @date 2021-04-22
**/
public interface DmInformationService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmInformationQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmInformationDto>
    */
    List<DmInformationDto> queryAll(DmInformationQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmInformationDto
     */
    DmInformationDto findById(Long id);

    /**
    * 创建
    * @param resources /
    * @return DmInformationDto
    */
    DmInformationDto create(DmInformation resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmInformation resources,String ... statusList);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmInformationDto> all, HttpServletResponse response) throws IOException;

     DmInformation toEntity(DmInformationDto dto);

    void updateTask(DmInformation resources);

    /**
     * todo 添加审批人
     * @param resources
     */
    void addApprovalUser(DmInformation resources);

    /**
     * todo 审批
     * @param resources
     */
    void approval(DmInformation resources);

    /**
     * todo 撤回
     * @param id
     */
    void ch(Long id);

    /**
     * 查询数据分页
     * @param pageable 分页参数
     * @return Map<String,Object>
     */
    Map<String,Object> findAllDSP(Pageable pageable);

}