/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.informationApproval.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.informationApproval.domain.DmInformationApproval;
import me.zhengjie.modules.dm.informationApproval.service.DmInformationApprovalService;
import me.zhengjie.modules.dm.informationApproval.service.dto.DmInformationApprovalQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author xwx
* @date 2021-04-23
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "dmInformationApproval管理")
@RequestMapping("/api/dmInformationApproval")
public class DmInformationApprovalController {

    private final DmInformationApprovalService dmInformationApprovalService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmInformationApproval:list')")
    public void download(HttpServletResponse response, DmInformationApprovalQueryCriteria criteria) throws IOException {
        dmInformationApprovalService.download(dmInformationApprovalService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询dmInformationApproval")
    @ApiOperation("查询dmInformationApproval")
    @PreAuthorize("@el.check('dmInformationApproval:list')")
    public ResponseEntity<Object> query(DmInformationApprovalQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmInformationApprovalService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增dmInformationApproval")
    @ApiOperation("新增dmInformationApproval")
    @PreAuthorize("@el.check('dmInformationApproval:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmInformationApproval resources){
        return new ResponseEntity<>(dmInformationApprovalService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改dmInformationApproval")
    @ApiOperation("修改dmInformationApproval")
    @PreAuthorize("@el.check('dmInformationApproval:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmInformationApproval resources){
        dmInformationApprovalService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除dmInformationApproval")
    @ApiOperation("删除dmInformationApproval")
    @PreAuthorize("@el.check('dmInformationApproval:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmInformationApprovalService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}