/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.informationApproval.service;

import me.zhengjie.modules.dm.informationApproval.domain.DmInformationApproval;
import me.zhengjie.modules.dm.informationApproval.service.dto.DmInformationApprovalDto;
import me.zhengjie.modules.dm.informationApproval.service.dto.DmInformationApprovalQueryCriteria;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author xwx
* @date 2021-04-23
**/
public interface DmInformationApprovalService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmInformationApprovalQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmInformationApprovalDto>
    */
    List<DmInformationApprovalDto> queryAll(DmInformationApprovalQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmInformationApprovalDto
     */
    DmInformationApprovalDto findById(Long id);

    /**
    * 创建
    * @param resources /
    * @return DmInformationApprovalDto
    */
    DmInformationApprovalDto create(DmInformationApproval resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmInformationApproval resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmInformationApprovalDto> all, HttpServletResponse response) throws IOException;

    /**
     * todo 刷新任务信息
     * @param infoId
     */
    void reset (Long infoId);

    /**
     * todo 刷新任务信息
     * @param infoId
     * @param approvalUser
     * @param taskId
     * @param procInstId
     */
    void resetTask (Long infoId,String approvalUser,String taskId,String procInstId);

    List<DmInformationApproval> findInfoByReqAndUser(Long infoId,String approvalUser);
}