/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.plateNumber.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.plateNumber.domain.DmPlateNumber;
import me.zhengjie.modules.dm.plateNumber.service.DmPlateNumberService;
import me.zhengjie.modules.dm.plateNumber.service.dto.DmPlateNumberQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Zheng Jie
* @date 2021-11-27
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/plateNumber管理")
@RequestMapping("/api/dmPlateNumber")
public class DmPlateNumberController {

    private final DmPlateNumberService dmPlateNumberService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmPlateNumber:list')")
    public void download(HttpServletResponse response, DmPlateNumberQueryCriteria criteria) throws IOException {
        dmPlateNumberService.download(dmPlateNumberService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/plateNumber")
    @ApiOperation("查询/api/plateNumber")
    @PreAuthorize("@el.check('dmPlateNumber:list')")
    public ResponseEntity<Object> query(DmPlateNumberQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmPlateNumberService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/plateNumber")
    @ApiOperation("新增/api/plateNumber")
    @PreAuthorize("@el.check('dmPlateNumber:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmPlateNumber resources){
        return new ResponseEntity<>(dmPlateNumberService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/plateNumber")
    @ApiOperation("修改/api/plateNumber")
    @PreAuthorize("@el.check('dmPlateNumber:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmPlateNumber resources){
        dmPlateNumberService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/plateNumber")
    @ApiOperation("删除/api/plateNumber")
    @PreAuthorize("@el.check('dmPlateNumber:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmPlateNumberService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}