/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.roomDevice.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.roomDevice.domain.DmRoomDevice;
import me.zhengjie.modules.dm.roomDevice.service.DmRoomDeviceService;
import me.zhengjie.modules.dm.roomDevice.service.dto.DmRoomDeviceQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Zheng Jie
* @date 2021-11-27
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/dmPlateNumber管理")
@RequestMapping("/api/dmRoomDevice")
public class DmRoomDeviceController {

    private final DmRoomDeviceService dmRoomDeviceService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmRoomDevice:list')")
    public void download(HttpServletResponse response, DmRoomDeviceQueryCriteria criteria) throws IOException {
        dmRoomDeviceService.download(dmRoomDeviceService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/dmPlateNumber")
    @ApiOperation("查询/api/dmPlateNumber")
    @PreAuthorize("@el.check('dmRoomDevice:list')")
    public ResponseEntity<Object> query(DmRoomDeviceQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmRoomDeviceService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/dmPlateNumber")
    @ApiOperation("新增/api/dmPlateNumber")
    @PreAuthorize("@el.check('dmRoomDevice:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmRoomDevice resources){
        return new ResponseEntity<>(dmRoomDeviceService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/dmPlateNumber")
    @ApiOperation("修改/api/dmPlateNumber")
    @PreAuthorize("@el.check('dmRoomDevice:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmRoomDevice resources){
        dmRoomDeviceService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/dmPlateNumber")
    @ApiOperation("删除/api/dmPlateNumber")
    @PreAuthorize("@el.check('dmRoomDevice:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmRoomDeviceService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}