/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.system.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.system.domain.DmSystem;
import me.zhengjie.modules.dm.system.service.DmSystemService;
import me.zhengjie.modules.dm.system.service.dto.DmSystemQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author Li Rui
* @date 2021-11-30
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "子系统管理")
@RequestMapping("/api/dmSystem")
public class DmSystemController {

    private final DmSystemService dmSystemService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmSystem:list')")
    public void download(HttpServletResponse response, DmSystemQueryCriteria criteria) throws IOException {
        dmSystemService.download(dmSystemService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询子系统")
    @ApiOperation("查询子系统")
    @PreAuthorize("@el.check('dmSystem:list')")
    public ResponseEntity<Object> query(DmSystemQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmSystemService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增子系统")
    @ApiOperation("新增子系统")
    @PreAuthorize("@el.check('dmSystem:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmSystem resources){
        return new ResponseEntity<>(dmSystemService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改子系统")
    @ApiOperation("修改子系统")
    @PreAuthorize("@el.check('dmSystem:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmSystem resources){
        dmSystemService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除子系统")
    @ApiOperation("删除子系统")
    @PreAuthorize("@el.check('dmSystem:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Integer[] ids) {
        dmSystemService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @Log("根据用户查询子系统")
    @ApiOperation("根据用户查询子系统")
    @GetMapping(value = "/queryByUserId")
    public ResponseEntity<Object> queryByUserId(String userId){
        return new ResponseEntity<>(dmSystemService.findByUserId(userId),HttpStatus.OK);
    }
}