/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.system.service.impl;

import me.zhengjie.modules.dm.system.domain.DmSystem;
import me.zhengjie.utils.ValidationUtil;
import me.zhengjie.utils.FileUtil;
import lombok.RequiredArgsConstructor;
import me.zhengjie.modules.dm.system.repository.DmSystemRepository;
import me.zhengjie.modules.dm.system.service.DmSystemService;
import me.zhengjie.modules.dm.system.service.dto.DmSystemDto;
import me.zhengjie.modules.dm.system.service.dto.DmSystemQueryCriteria;
import me.zhengjie.modules.dm.system.service.mapstruct.DmSystemMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import me.zhengjie.utils.PageUtil;
import me.zhengjie.utils.QueryHelp;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.LinkedHashMap;

/**
* @website https://el-admin.vip
* @description 服务实现
* @author Li Rui
* @date 2021-11-30
**/
@Service
@RequiredArgsConstructor
public class DmSystemServiceImpl implements DmSystemService {

    private final DmSystemRepository dmSystemRepository;
    private final DmSystemMapper dmSystemMapper;

    @Override
    public Map<String,Object> queryAll(DmSystemQueryCriteria criteria, Pageable pageable){
        Page<DmSystem> page = dmSystemRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder),pageable);
        return PageUtil.toPage(page.map(dmSystemMapper::toDto));
    }

    @Override
    public List<DmSystemDto> queryAll(DmSystemQueryCriteria criteria){
        return dmSystemMapper.toDto(dmSystemRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root,criteria,criteriaBuilder)));
    }

    @Override
    @Transactional
    public DmSystemDto findById(Integer id) {
        DmSystem dmSystem = dmSystemRepository.findById(id).orElseGet(DmSystem::new);
        ValidationUtil.isNull(dmSystem.getId(),"DmSystem","id",id);
        return dmSystemMapper.toDto(dmSystem);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DmSystemDto create(DmSystem resources) {
        return dmSystemMapper.toDto(dmSystemRepository.save(resources));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DmSystem resources) {
        DmSystem dmSystem = dmSystemRepository.findById(resources.getId()).orElseGet(DmSystem::new);
        ValidationUtil.isNull( dmSystem.getId(),"DmSystem","id",resources.getId());
        dmSystem.copy(resources);
        dmSystemRepository.save(dmSystem);
    }

    @Override
    public void deleteAll(Integer[] ids) {
        for (Integer id : ids) {
            dmSystemRepository.deleteById(id);
        }
    }

    @Override
    public void download(List<DmSystemDto> all, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (DmSystemDto dmSystem : all) {
            Map<String,Object> map = new LinkedHashMap<>();
            map.put("子系统名称", dmSystem.getSystemName());
            map.put("子系统地址", dmSystem.getSystemUrl());
            map.put("子系统图标id", dmSystem.getStorage());
            map.put("子系统管理员", dmSystem.getDmUser());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }

    @Override
    public List<DmSystem> findByUserId(String userId) {
        return dmSystemRepository.findByUserId(userId);
    }

}