/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.user.service;

import me.zhengjie.modules.dm.user.domain.DmUser;
import me.zhengjie.modules.dm.user.service.dto.DmUserDto;
import me.zhengjie.modules.dm.user.service.dto.DmUserQueryCriteria;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author shiguangji
* @date 2021-03-09
**/
public interface DmUserService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmUserQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmUserDto>
    */
    List<DmUserDto> queryAll(DmUserQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmUserDto
     */
    DmUserDto findById(String id);

    /**
    * 创建
    * @param resources /
    * @return DmUserDto
    */
    DmUserDto create(DmUser resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmUser resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(String[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmUserDto> all, HttpServletResponse response) throws IOException;

    /**
     * 查询未绑定账号的用户
     * @param userId
     * @return
     */
    List<DmUser> findNotBindUser(String userId);

    /**
     * 根据用户id删除用户系统表数据
     * @param userId /
     */
    void deleteUserSystem(String userId);

    /**
     * 新增用户系统数据
     * @param userId /
     * @param systemId
     */
    void createUserSystem(String userId, Integer systemId);

    /**
     * 根据用户id删除用户app菜单表数据
     * @param userId /
     */
    void deleteUserAppMenu(String userId);

    /**
     * 新增用户app菜单数据
     * @param userId
     * @param appMenuId
     */
    void createUserAppMenu(String userId, Integer appMenuId);

    /**
     * 查询所有数据不分页
     * @return List<DmUserDto>
     */
    List<Map<String,String>> queryAll();
}