/*
 *  Copyright 2019-2020 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package me.zhengjie.repository;

import me.zhengjie.domain.LocalStorage;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;

import java.util.Map;

/**
* @author Zheng Jie
* @date 2019-09-05
*/
public interface LocalStorageRepository extends JpaRepository<LocalStorage, Long>, JpaSpecificationExecutor<LocalStorage> {

    /**
     * 根据文件名删除
     * @param oldName
     * @return
     */
    @Modifying
    @Query(value = "DELETE FROM tool_local_storage WHERE real_name = :oldName",nativeQuery = true)
    int deleteByUrl(@Param("oldName") String oldName);

    /**
     * 根据文件名查询
     * @param avatarName 文件名称
     * @return
     */
    @Transactional
    @Query(value = "SELECT url FROM tool_local_storage WHERE real_name = :avatarName ",nativeQuery = true)
    Map<String, String> findByRealName(@Param("avatarName") String avatarName);
}